#include "os.h"
#include "descriptor_tables.h"

// ASM functions in flush.asm
extern void gdt_flush(ULONG);
extern void idt_flush(ULONG);
extern void tss_flush();

// Internal function prototypes.
void write_tss(int,USHORT,ULONG);

gdt_entry_t gdt_entries[6];
gdt_ptr_t   gdt_ptr;
idt_entry_t idt_entries[256];
idt_ptr_t   idt_ptr;
tss_entry_t tss_entry;


// Initialise our task state segment structure.
void write_tss(int num, USHORT ss0, ULONG esp0)
{
    // Firstly, let's compute the base and limit of our entry into the GDT.
    ULONG base = (ULONG) &tss_entry;
    ULONG limit = sizeof(tss_entry); //http://forum.osdev.org/viewtopic.php?f=1&t=19819&p=155587&hilit=tss_entry#p155587

    // Now, add our TSS descriptor's address to the GDT.
    gdt_set_gate(num, base, limit, 0xE9, 0x00);

    // Ensure the descriptor is initially zero.
    k_memset(&tss_entry, 0, sizeof(tss_entry));

    tss_entry.ss0  = ss0;  // Set the kernel stack segment.
    tss_entry.esp0 = esp0; // Set the kernel stack pointer.

    // Here we set the cs, ss, ds, es, fs and gs entries in the TSS. These specify what
    // segments should be loaded when the processor switches to kernel mode. Therefore
    // they are just our normal kernel code/data segments - 0x08 and 0x10 respectively,
    // but with the last two bits set, making 0x0b and 0x13. The setting of these bits
    // sets the RPL (requested privilege level) to 3, meaning that this TSS can be used
    // to switch to kernel mode from ring 3.
    tss_entry.cs   = 0x0b;
    tss_entry.ss = tss_entry.ds = tss_entry.es = tss_entry.fs = tss_entry.gs = 0x13;
}

void set_kernel_stack(ULONG stack)
{
    tss_entry.esp0 = stack;
}
