#include "syscall.h"

void syscall_handler(struct regs* regs);

DEFN_SYSCALL1(puts, 0, UCHAR*);

static void* syscalls[1] = { &puts };
ULONG num_syscalls = 1;

void syscall_handler(struct regs* regs)
{
    // Firstly, check if the requested syscall number is valid. The syscall number is found in EAX.

    if (regs->eax >= num_syscalls)
        return;

    // Get the required syscall location.
    void *location = syscalls[regs->eax];

    // We don't know how many parameters the function wants, so we just
    // push them all onto the stack in the correct order. The function will
    // use all the parameters it wants, and we can pop them all back off afterwards.
    int ret;
    asm volatile (" \
      push %1; \
      push %2; \
      push %3; \
      push %4; \
      push %5; \
      call *%6; \
      pop %%ebx; \
      pop %%ebx; \
      pop %%ebx; \
      pop %%ebx; \
      pop %%ebx; \
    " : "=a" (ret) : "r" (regs->edi), "r" (regs->esi), "r" (regs->edx), "r" (regs->ecx), "r" (regs->ebx), "r" (location));
    regs->eax = ret;
}

